<?php

class nc_payment_system_levipay extends nc_payment_system
{
	protected $automatic = TRUE;
	
	const ERROR_MERCHANT_URL_IS_NOT_VALID = NETCAT_MODULE_PAYMENT_LEVIPAY_ERROR_MERCHANT_URL_IS_NOT_VALID;
	const ERROR_MERCHANT_ID_IS_NOT_VALID = NETCAT_MODULE_PAYMENT_LEVIPAY_ERROR_MERCHANT_ID_IS_NOT_VALID;
	const ERROR_SECRET_KEY_IS_NOT_VALID = NETCAT_MODULE_PAYMENT_LEVIPAY_ERROR_SECRET_KEY_IS_NOT_VALID;
	const MSG_NOT_VALID_IP = NETCAT_MODULE_PAYMENT_LEVIPAY_MSG_NOT_VALID_IP;
	const MSG_VALID_IP = NETCAT_MODULE_PAYMENT_LEVIPAY_MSG_VALID_IP;
	const MSG_THIS_IP = NETCAT_MODULE_PAYMENT_LEVIPAY_MSG_THIS_IP;
	const MSG_HASHES_NOT_EQUAL = NETCAT_MODULE_PAYMENT_LEVIPAY_MSG_HASHES_NOT_EQUAL;
	const MSG_WRONG_AMOUNT = NETCAT_MODULE_PAYMENT_LEVIPAY_MSG_WRONG_AMOUNT;
	const MSG_WRONG_CURRENCY = NETCAT_MODULE_PAYMENT_LEVIPAY_MSG_WRONG_CURRENCY;
	const MSG_WRONG_ORDER_PAYEED = NETCAT_MODULE_PAYMENT_LEVIPAY_MSG_WRONG_ORDER_PAYEED;
	const MSG_STATUS_FAIL = NETCAT_MODULE_PAYMENT_LEVIPAY_MSG_STATUS_FAIL;
	const MSG_ERR_REASONS = NETCAT_MODULE_PAYMENT_LEVIPAY_MSG_ERR_REASONS;
	const MSG_SUBJECT = NETCAT_MODULE_PAYMENT_LEVIPAY_MSG_SUBJECT;

    protected $accepted_currencies = array('RUB', 'RUR');

    protected $settings = array(
        'MERCHANT_URL' => 'https://levipay.com/pay/',
        'MERCHANT_ID' => null,
        'SECRET_KEY1' => null,
        'SECRET_KEY2' => null,
		'LOG_FILE' => '/levipay.log',
		'IP_FILTER' => null,
		'ADMIN_EMAIL' => "Ваш email"
    );

    protected $request_parameters = array();

    protected $callback_response = array(
        'MERCHANT_ID' => null,
        'AMOUNT' => null,
	    'intid' => null,
	    'MERCHANT_ORDER_ID' => null,
		'P_EMAIL' => null,
	    'P_PHONE' => null,
	    'CUR_ID' => null,
		'SIGN' => null,
    );

    public function execute_payment_request(nc_payment_invoice $invoice) 
	{
		$m_url = $this->get_setting('MERCHANT_URL');
		$m_shop = $this->get_setting('MERCHANT_ID');
		$m_key = $this->get_setting('SECRET_KEY1');
		$m_orderid = $invoice->get_id();
		$m_amount = number_format($invoice->get_amount("%0.2F"), 2, '.', '');
		$m_desc = base64_encode($invoice->get_description());

		$sign = md5( implode(":", array(
            $m_shop,
            $m_amount,
            $m_key,
            $m_orderid,
        )));

        ob_end_clean();
		
        $form = "
            <html>
              <body>
                    <form action='" . $m_url . "' method='GET'>" .
                    $this->make_inputs(array(
                        'm' => $m_shop,
                        'oa' => $m_amount,
                        'o' => $m_orderid,
                        's' => $sign
                    )) . "
                </form>
                <script>
                  document.forms[0].submit();
                </script>
              </body>
            </html>";
			
        echo $form;
        exit;
    }

    public function on_response(nc_payment_invoice $invoice = null) 
	{
    }

    public function validate_payment_request_parameters() 
	{
		$m_url = $this->get_setting('MERCHANT_URL');
		$m_id = $this->get_setting('MERCHANT_ID');
		$m_key = $this->get_setting('SECRET_KEY2');
		
        if (empty($m_url)) 
		{
            $this->add_error(nc_payment_system_levipay::ERROR_MERCHANT_URL_IS_NOT_VALID);
        }
		
		if (empty($m_id)) 
		{
            $this->add_error(nc_payment_system_levipay::ERROR_MERCHANT_ID_IS_NOT_VALID);
        }
		
		if (empty($m_key))
		{
            $this->add_error(nc_payment_system_levipay::ERROR_SECRET_KEY_IS_NOT_VALID);
        }
    }

    public function validate_payment_callback_response(nc_payment_invoice $invoice = null) 
	{
		$m_operation_id = $this->get_response_value('intid');
		$m_sign = $this->get_response_value('SIGN');
		
		if (isset($m_operation_id) && isset($m_sign))
		{
			$err = false;
			$message = '';

			// запись логов

            $log_text =
                "--------------------------------------------------------\n" .
                "id you	shoop   	" . $this->get_response_value("MERCHANT_ID") . "\n" .
                "amount				" . $this->get_response_value("AMOUNT") . "\n" .
                "kassa operation id " . $this->get_response_value("intid") . "\n" .
                "merchant order id	" . $this->get_response_value("MERCHANT_ORDER_ID") . "\n" .
                "e-mail client		" . $this->get_response_value("P_EMAIL") . "\n" .
                "phone client    	" . $this->get_response_value("P_PHONE") . "\n" .
                "currency			" . $this->get_response_value("CUR_ID") . "\n" .
                "sign				" . $this->get_response_value("SIGN") . "\n\n";

			$log_file = $this->get_setting('LOG_FILE');
			
			if (!empty($log_file))
			{
				file_put_contents($_SERVER['DOCUMENT_ROOT'] . $log_file, $log_text, FILE_APPEND);
			}
			
			// проверка цифровой подписи и ip
			
			$sign_hash = md5( implode(":", array(
				$this->get_response_value('MERCHANT_ID'),
				$this->get_response_value('AMOUNT'),
				$this->get_setting('SECRET_KEY'),
				$this->get_response_value('MERCHANT_ORDER_ID'),

			)));
			

			$sIP = str_replace(' ', '', $this->get_setting('IP_FILTER'));
            $valid_ip = true;
            if ( ! empty( $sIP ) ) {

                $ip_filter_arr = explode(',', $sIP);
                $this_ip  = (isset($_SERVER['HTTP_X_REAL_IP'])) ? $_SERVER['HTTP_X_REAL_IP'] : $_SERVER['REMOTE_ADDR'];

                foreach ( $ip_filter_arr as $key => $value ) {
                    $ip_filter_arr[ $key ] = ip2long( $value );
                }

                if ( ! in_array( ip2long($this_ip), $ip_filter_arr)) {
                    $valid_ip = false;
                }

            }
			
			if (!$valid_ip)
			{
				$message .= nc_payment_system_levipay::MSG_NOT_VALID_IP . "\n" .
				nc_payment_system_levipay::MSG_VALID_IP . $sIP . "\n" .
				nc_payment_system_levipay::MSG_THIS_IP . $_SERVER['REMOTE_ADDR'] . "\n";
				$err = true;
			}

			if (!$err)
			{
				$order_amount = number_format($invoice->get_amount(), 2, '.', '');
				
				// проверка суммы
				
				if ($this->get_response_value('AMOUNT') != $order_amount)
				{
					$message .= nc_payment_system_levipay::MSG_WRONG_AMOUNT . "\n";
					$err = true;
				}

				// проверка статуса
				
				if (!$err)
				{
					if ($this->get_response_value('SIGN') != $sign_hash) {

                            if ($invoice->get('status') != 6) {
                                $invoice->set('status', nc_payment_invoice::STATUS_SUCCESS);
                                $invoice->save();
                                $this->on_payment_success($invoice);
                            }

                    }else {

							$invoice->set('status', nc_payment_invoice::STATUS_CALLBACK_ERROR);
							$invoice->save();
                            $message .= nc_payment_system_levipay::MSG_HASHES_NOT_EQUAL . "\n";
							$err = true;
					}
				}
			}
			
			if ($err)
			{
				$to = $this->get_setting('ADMIN_EMAIL');

				if (!empty($to))
				{
					$message = nc_payment_system_levipay::MSG_ERR_REASONS . "\n\n" . $message . "\n" . $log_text;
					$headers = "From: no-reply@" . $_SERVER['HTTP_HOST'] . "\r\n" . 
					"Content-type: text/plain; charset=utf-8 \r\n";
					mail($to, nc_payment_system_levipay::MSG_SUBJECT, $message, $headers);
				}
				
				echo ($this->get_response_value('MERCHANT_ORDER_ID') . ' |error| ' . $message);
			}
			else
			{
				echo 'YES';
			}
		}
    }

    public function load_invoice_on_callback() 
	{
        return $this->load_invoice($this->get_response_value('MERCHANT_ORDER_ID'));
    }
}